@@ -2,6 +2,7 @@
  * @file common.c common routines for Liferea
  * 
  * Copyright (C) 2003, 2004 Lars Lindner <lars.lindner@gmx.net>
+ * Copyright (C) 2004       Nathan J. Conrad <t98502@users.sourceforge.net>
  * Copyright (C) 2004       Karl Soderstrom <ks@xanadunet.net>
  *
  * parts of the RFC822 timezone decoding were taken from the gmime 
@@ -28,13 +29,19 @@
  */
 
 #define _XOPEN_SOURCE /* glibc2 needs this (man strptime) */
-#include <libxml/xmlmemory.h>
+#include <config.h>
+
+#include <libxml/xmlerror.h>
+#include <libxml/uri.h>
 #include <libxml/parser.h>
-#include <libxml/parserInternals.h>
+#include <libxml/HTMLparser.h>
 #include <glib.h>
 #include <sys/stat.h>
 #include <string.h>
 #include <locale.h>
+#include <time.h>
+#include <stdlib.h>
+#include <ctype.h>
 #include "support.h"
 #include "common.h"
 #include "conf.h"
@@ -48,6 +55,22 @@ static gchar *lifereaUserPath = NULL;
 
 gchar * convertCharSet(gchar * from_encoding, gchar * to_encoding, gchar * string);
 
+void addToHTMLBufferFast(gchar **buffer, const gchar *string) {
+	
+	if(NULL == string)
+		return;
+	
+	if(NULL != *buffer) {
+		int oldlength = strlen(*buffer);
+		int newlength = strlen(string);
+		int allocsize = (((oldlength+newlength+1)/512)+1)*512; /* Round up to nearest 512 KB */
+		*buffer = g_realloc(*buffer, allocsize);
+		g_memmove(&((*buffer)[oldlength]), string, newlength+1 );
+	} else {
+		*buffer = g_strdup(string);
+	}
+}
+
 void addToHTMLBuffer(gchar **buffer, const gchar *string) {
 	
 	if(NULL == string)
@@ -56,7 +79,7 @@ void addToHTMLBuffer(gchar **buffer, const gchar *string) {
 	if(NULL != *buffer) {
 		int oldlength = strlen(*buffer);
 		int newlength = strlen(string);
-		int allocsize = oldlength + newlength + 1; // Round up to nearest 512 KB is a bad idea since it wastes memory.
+		int allocsize = (oldlength+newlength+1);
 		*buffer = g_realloc(*buffer, allocsize);
 		g_memmove(&((*buffer)[oldlength]), string, newlength+1 );
 	} else {
@@ -70,13 +93,17 @@ void addToHTMLBuffer(gchar **buffer, const gchar *string) {
 gchar * convertCharSet(gchar * from_encoding, gchar * to_encoding, gchar * string) {
 	gint	bw, br;
 	gchar	*new = NULL;
-	
+	GError *err = NULL;
+
 	if(NULL == from_encoding)
 		from_encoding = standard_encoding;
 		
 	if(NULL != string) {		
-		new = g_convert(string, strlen(string), to_encoding, from_encoding, &br, &bw, NULL);
-		
+		new = g_convert(string, strlen(string), to_encoding, from_encoding, &br, &bw, &err);
+		if (err != NULL) {
+			g_warning("error converting character set: %s\n", err->message);
+			g_error_free (err);
+		}
 		if(NULL != new)
 			g_free(string);
 		else
@@ -88,14 +115,14 @@ gchar * convertCharSet(gchar * from_encoding, gchar * to_encoding, gchar * strin
 	return new;
 }
 
-gchar * convertToHTML(gchar * string) { return convertCharSet("UTF-8", "HTML", string); }
+gchar * convertToHTML(gchar * string) { return string; } /* FIXME: BROKEN! return convertCharSet("UTF-8", "HTML", string); } */
 
 /* Conversion function which should be applied to all read XML strings, 
    to ensure proper UTF8. This is because we use libxml2 in recovery
    mode which can produce invalid UTF-8. 
    
-   The valid or a corrected string is returned, the original XML 
-   string is freed. */
+   The valid or a corrected string is returned. The original XML 
+   string is modified (FIXME: not sure if this is good). */
 gchar * utf8_fix(xmlChar *string) {
 	const gchar	*invalid_offset;
 
@@ -121,13 +148,21 @@ gchar * utf8_fix(xmlChar *string) {
 	return string;
 }
 
-gchar * extractHTMLNode(xmlNodePtr cur) {
-	xmlBufferPtr	buf = NULL;
+gchar * extractHTMLNode(xmlNodePtr cur, gboolean children) {
+	xmlBufferPtr	buf;
 	gchar		*result = NULL;
-	
+
 	buf = xmlBufferCreate();
-	
-	if(-1 != xmlNodeDump(buf, cur->doc, cur, 0, 0))
+	if(children) {
+		cur = cur->xmlChildrenNode;
+		while (cur != NULL) {
+			xmlNodeDump(buf, cur->doc, cur, 0, 0);
+			cur = cur->next;
+		}
+	} else {
+		xmlNodeDump(buf, cur->doc, cur, 0, 0);
+	}
+	if(xmlBufferLength(buf) > 0)
 		result = xmlCharStrdup(xmlBufferContent(buf));
 
 	xmlBufferFree(buf);
@@ -213,43 +248,84 @@ typedef struct errorCtxt {
  * @param	ctxt	error context
  * @param	msg	printf like format string 
  */
-void bufferParseError(void *ctxt, const gchar * msg, ...) {
+static void bufferParseError(void *ctxt, const gchar * msg, ...) {
 	va_list		params;
 	errorCtxtPtr	errors = (errorCtxtPtr)ctxt;
-	gchar		*oldmsg;
 	gchar		*newmsg;
 	gchar		*tmp;
 
 	g_assert(NULL != errors);
-
-	if(MAX_PARSE_ERROR_LINES + 1 >= errors->errorCount++) {
-		oldmsg = errors->buffer;
+	
+	if(MAX_PARSE_ERROR_LINES > errors->errorCount++) {
 		
 		va_start(params, msg);
 		newmsg = g_strdup_vprintf(msg, params);
 		va_end(params);
 
 		g_assert(NULL != newmsg);
+		newmsg = utf8_fix(newmsg);
 		tmp = g_markup_escape_text(newmsg, -1);
 		g_free(newmsg);
 		newmsg = tmp;
+	
+		addToHTMLBufferFast(&errors->buffer, "<pre>");
+		addToHTMLBufferFast(&errors->buffer, newmsg);
+		addToHTMLBufferFast(&errors->buffer, "</pre>");
 
-		if(NULL != oldmsg) 
-			errors->buffer = g_strdup_printf("%s<br>%s", oldmsg, newmsg);
-		else
-			errors->buffer = g_strdup(newmsg);
-
-		g_free(oldmsg);
 		g_free(newmsg);
 	}
 	
 	if(MAX_PARSE_ERROR_LINES == errors->errorCount) {
-		newmsg = g_strdup_printf("%s<br>%s", errors->buffer, _("[Parser error output was truncated!]"));
+		newmsg = g_strdup_printf("%s<br/>%s", errors->buffer, _("[Parser error output was truncated!]"));
 		g_free(errors->buffer);
 		errors->buffer = newmsg;
 	}
 }
 
+static GHashTable *extra_entities = NULL;
+
+/* and I thought writing such functions is evil... */
+xmlEntityPtr common_process_entities(void *ctxt, const xmlChar *name) {
+	xmlEntityPtr	entity;
+	gchar		*text;
+	
+	entity = xmlGetPredefinedEntity(name);
+	if(NULL == entity) {
+		if(NULL == extra_entities) {
+			extra_entities = g_hash_table_new(g_str_hash, g_str_equal);
+			/* attention! ensure UTF-8 for the following definitions! */
+			g_hash_table_insert(extra_entities, "nbsp", " ");
+			g_hash_table_insert(extra_entities, "auml", "ä");
+			g_hash_table_insert(extra_entities, "Auml", "Ä");
+			g_hash_table_insert(extra_entities, "uuml", "ü");
+			g_hash_table_insert(extra_entities, "Uuml", "Ü");
+			g_hash_table_insert(extra_entities, "ouml", "ö");
+			g_hash_table_insert(extra_entities, "Ouml", "Ö");
+			g_hash_table_insert(extra_entities, "szlig", "ß");
+			g_hash_table_insert(extra_entities, "copy", "©");
+			g_hash_table_insert(extra_entities, "hellip", "…");
+			g_hash_table_insert(extra_entities, "laquo", "«");
+			g_hash_table_insert(extra_entities, "raquo", "»");
+			// FIXME: someone who want's to add all other HTML entity definitions?
+			// or better somehow load a DTD and serve entities when necessary...
+		}
+		if(NULL != (text = g_hash_table_lookup(extra_entities, name))) {
+			/* returning as faked predefined entity... */
+			entity = (xmlEntityPtr)g_new0(xmlEntity, 1);
+			entity->type = XML_ENTITY_DECL;
+			entity->name = name;
+			entity->orig = text;	/* ??? */
+			entity->content = text;
+			entity->length = g_utf8_strlen(text, -1);
+			entity->etype = XML_INTERNAL_PREDEFINED_ENTITY;
+		}
+	}
+	if(NULL == entity) {		
+		g_print("unsupported entity: %s\n", name);
+	}
+	return entity;
+}
+
 /**
  * Common function to create a XML DOM object from a given
  * XML buffer. This function sets up a parser context,
@@ -261,55 +337,59 @@ void bufferParseError(void *ctxt, const gchar * msg, ...) {
  * errors. 
  *
  * @param data		XML source
+ * @param dataLength the length of the data string in bytes
  * @param errormsg	error buffer
  *
  * @return XML document
  */
-xmlDocPtr parseBuffer(gchar *data, gchar **errormsg) {
+xmlDocPtr parseBuffer(gchar *data, size_t dataLength, gchar **errormsg) {
+	xmlParserCtxtPtr	ctxt;
 	errorCtxtPtr		errors;
-	xmlParserCtxtPtr	parser;
-	xmlDocPtr		doc;
-	gint			length;
+	xmlDocPtr               doc;
 	
 	g_assert(NULL != data);
-
-	/* xmlCreateMemoryParserCtxt() doesn't like no data */
-	if(0 == (length = strlen(data))) {
+	
+	/* we don't like no data */
+	if(0 == dataLength) {
 		g_warning("parseBuffer(): Empty input!\n");
 		*errormsg = g_strdup("parseBuffer(): Empty input!\n");
 		return NULL;
 	}
 	
-	if(NULL != (parser = xmlCreateMemoryParserCtxt(data, length))) {
-		parser->recovery = 1;
-		errors = g_new0(struct errorCtxt, 1);
-		xmlSetGenericErrorFunc(errors, (xmlGenericErrorFunc)bufferParseError);
-		xmlParseDocument(parser);	/* ignore returned errors */
-		doc = parser->myDoc;
-		xmlFreeParserCtxt(parser);		
-		*errormsg = errors->buffer;
-		g_free(errors);
- /* This seems to reset the errorfunc to he default, so that the
-    GtkHTML2 module is not unhappy because it also tries to call the
-    errorfunc on occasion. */
-		xmlSetGenericErrorFunc(NULL, NULL);
-	} else {
-		g_warning("parseBuffer(): Could not create parsing context!\n");
-		*errormsg = g_strdup("parseBuffer(): Could not create parsing context!\n");
-		return NULL;
+	errors = g_new0(struct errorCtxt, 1);
+	xmlSetGenericErrorFunc(errors, (xmlGenericErrorFunc)bufferParseError);
+	ctxt = xmlNewParserCtxt();
+	ctxt->sax->getEntity = common_process_entities;
+	doc = xmlSAXParseMemory(ctxt->sax, data, dataLength, /* recovery = */ TRUE);
+	if (doc == NULL) {
+		g_warning("xmlReadMemory: Could not parse document!\n");
+		*errormsg = g_strdup_printf(_("xmlReadMemory(): Could not parse document:\n%s%s"), 
+		                            errors->buffer != NULL ? errors->buffer : "",
+		                            errors->buffer != NULL ? "\n" : "");
+		g_free(errors->buffer);
+		errors->buffer = *errormsg;
 	}
 	
+	/* This seems to reset the errorfunc to its default, so that the
+	   GtkHTML2 module is not unhappy because it also tries to call the
+	   errorfunc on occasion. */
+	xmlSetGenericErrorFunc(NULL, NULL);
+	
+	*errormsg = errors->buffer;
+	g_free(errors);
+	xmlFreeParserCtxt(ctxt);
 	return doc;
 }
 
 /* converts a ISO 8601 time string to a time_t value */
 time_t parseISO8601Date(gchar *date) {
 	struct tm	tm;
-	time_t		t;
+	time_t		t, t2, offset = 0;
 	gboolean	success = FALSE;
-		
+	gchar *pos;
+	
 	memset(&tm, 0, sizeof(struct tm));
-
+	
 	/* we expect at least something like "2003-08-07T15:28:19" and
 	   don't require the second fractions and the timezone info
 
@@ -317,30 +397,113 @@ time_t parseISO8601Date(gchar *date) {
 	 */
 	 
 	/* full specified variant */
-	if(NULL != strptime((const char *)date, "%t%Y-%m-%dT%H:%M%t", &tm))
+	if(NULL != (pos = strptime((const char *)date, "%t%Y-%m-%dT%H:%M%t", &tm))) {
+		/* Parse seconds */
+		if (*pos == ':')
+			pos++;
+		if (isdigit(pos[0]) && !isdigit(pos[1])) {
+			tm.tm_sec = pos[0] - '0';
+			pos++;
+		} else if (isdigit(pos[0]) && isdigit(pos[1])) {
+			tm.tm_sec = 10*(pos[0]-'0') + pos[0] - '0';
+			pos +=2;
+		}
+		/* Parse timezone */
+		if (*pos == 'Z')
+			offset = 0;
+		else if ((*pos == '+' || *pos == '-') && isdigit(pos[1]) && isdigit(pos[2]) && strlen(pos) >= 3) {
+			offset = (10*(pos[1] - '0') + (pos[2] - '0')) * 60 * 60;
+			
+			if (pos[3] == ':' && isdigit(pos[4]) && isdigit(pos[5]))
+				offset +=  (10*(pos[4] - '0') + (pos[5] - '0')) * 60;
+			else if (isdigit(pos[3]) && isdigit(pos[4]))
+				offset +=  (10*(pos[3] - '0') + (pos[4] - '0')) * 60;
+			
+			offset *= (pos[0] == '+') ? 1 : -1;
+
+		}
 		success = TRUE;
 	/* only date */
-	else if(NULL != strptime((const char *)date, "%t%Y-%m-%d", &tm))
+	} else if(NULL != strptime((const char *)date, "%t%Y-%m-%d", &tm))
 		success = TRUE;
 	/* there were others combinations too... */
 
 	if(TRUE == success) {
 		if((time_t)(-1) != (t = mktime(&tm))) {
+			/* Correct for the local timezone*/
+			t = t - offset;
+			t2 = mktime(gmtime(&t));
+			t = t - (t2 - t);
+			
 			return t;
 		} else {
-			g_warning(_("internal error! time conversion error! mktime failed!\n"));
+			g_message(_("internal error! time conversion error! mktime failed!\n"));
 		}
 	} else {
-		g_message(_("Invalid ISO8601 date format! Ignoring <dc:date> information!\n"));				
+		g_message(_("Invalid ISO8601 date format! Ignoring <dc:date> information!\n"));
 	}
 	
 	return 0;
 }
 
+gchar *dayofweek[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
+gchar *months[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
+
+gchar *createRFC822Date(const time_t *time) {
+	struct tm *tm;
+
+	tm = gmtime(time); /* No need to free because it is statically allocated */
+	return g_strdup_printf("%s, %2d %s %4d %02d:%02d:%02d GMT", dayofweek[tm->tm_wday], tm->tm_mday,
+					   months[tm->tm_mon], 1900 + tm->tm_year, tm->tm_hour, tm->tm_min, tm->tm_sec);
+}
+
+/* this table of RFC822 timezones is from gmime-utils.c of the gmime API */
+static struct {
+	char *name;
+	int offset;
+} tz_offsets [] = {
+	{ "UT", 0 },
+	{ "GMT", 0 },
+	{ "EST", -500 },        /* these are all US timezones.  bloody yanks */
+	{ "EDT", -400 },
+	{ "CST", -600 },
+	{ "CDT", -500 },
+	{ "MST", -700 },
+	{ "MDT", -600 },
+	{ "PST", -800 },
+	{ "PDT", -700 },
+	{ "Z", 0 },
+	{ "A", -100 },
+	{ "M", -1200 },
+	{ "N", 100 },
+	{ "Y", 1200 }
+};
+/** @returns timezone offset in seconds */
+static time_t common_parse_rfc822_tz(unsigned char *token) {
+	int offset = 0;
+	const unsigned char *inptr = token;
+	
+	if (*inptr == '+' || *inptr == '-') {
+		offset = atoi (inptr);
+	} else {
+		int t;
+
+		if (*inptr == '(')
+			inptr++;
+
+		for (t = 0; t < 15; t++)
+			if (!strncmp (inptr, tz_offsets[t].name, strlen (tz_offsets[t].name)))
+				offset = tz_offsets[t].offset;
+	}
+	
+	return 60 * ((offset / 100) * 60 + (offset % 100));
+}
+
+
 /* converts a RFC822 time string to a time_t value */
 time_t parseRFC822Date(gchar *date) {
 	struct tm	tm;
-	time_t		t;
+	time_t		t, t2;
 	char 		*oldlocale;
 	char		*pos;
 	gboolean	success = FALSE;
@@ -367,19 +530,25 @@ time_t parseRFC822Date(gchar *date) {
 	else if(NULL != (pos = strptime((const char *)date, "%d %b %Y %T", &tm)))
 		success = TRUE;
 	
+	while(pos != NULL && *pos != '\0' && isspace((int)*pos))       /* skip whitespaces before timezone */
+		pos++;
+	
 	if(NULL != oldlocale) {
 		setlocale(LC_TIME, oldlocale);	/* and reset it again */
 		g_free(oldlocale);
 	}
 	
 	if(TRUE == success) {
-		if((time_t)(-1) != (t = mktime(&tm)))
+		if((time_t)(-1) != (t = mktime(&tm))) {
+			/* GMT time, with no daylight savings time
+			   correction. (Usually, there is no daylight savings
+			   time since the input is GMT.) */
+			t = t - common_parse_rfc822_tz(pos);
+			t2 = mktime(gmtime(&t));
+			t = t - (t2 - t);
 			return t;
-		else
+		} else
 			g_warning(_("internal error! time conversion error! mktime failed!\n"));
-	} else {
-
-		g_message(_("Invalid RFC822 date format! Ignoring <pubDate> information!\n"));
 	}
 	
 	return 0;
@@ -423,7 +592,7 @@ void initCachePath(void) {
 
 }
 
-gchar * getCachePath(void) {
+gchar * common_get_cache_path(void) {
 	
 	if(NULL == lifereaUserPath)
 		initCachePath();
@@ -431,16 +600,16 @@ gchar * getCachePath(void) {
 	return lifereaUserPath;
 }
 
-gchar * common_create_cache_filename( gchar *folder, gchar *key, gchar *extension) {
+gchar * common_create_cache_filename( const gchar *folder, const gchar *key, const gchar *extension) {
 	gchar *filename;
 
-	filename = g_strdup_printf("%s" G_DIR_SEPARATOR_S "%s%s%s%s%s", getCachePath(),
-						  (folder != NULL) ? folder : "",
-						  (folder != NULL) ? G_DIR_SEPARATOR_S : "",
-						  key,
-						  (extension != NULL)? "." : "",
-						  (extension != NULL)? extension : "");
-	
+	filename = g_strdup_printf("%s" G_DIR_SEPARATOR_S "%s%s%s%s%s", common_get_cache_path(),
+	                           (folder != NULL) ? folder : "",
+	                           (folder != NULL) ? G_DIR_SEPARATOR_S : "",
+	                           key,
+	                           (extension != NULL)? "." : "",
+	                           (extension != NULL)? extension : "");
+
 	return filename;
 }
 
@@ -451,11 +620,7 @@ static gchar * byte_to_hex(unsigned char nr) {
 	return result;
 }
 
-/* Encodes any UTF-8 string in uriString and returns a 
-   valid UTF-8 encoded HTTP URI. Note that the uriString will 
-   be freed. This function is actually used to generate Feedster
-   search feed URLs. */
-gchar * encodeURIString(gchar *uriString) {
+gchar * encode_uri_string(gchar *string) {
 	gchar		*newURIString;
 	gchar		*hex, *tmp = NULL;
 	int		i, j, len, bytes;
@@ -464,37 +629,25 @@ gchar * encodeURIString(gchar *uriString) {
 	   the characters bytewise and convert non-ASCII
 	   compatible chars to URI hexcodes */
 	newURIString = g_strdup("");
-	len = strlen(uriString);
+	len = strlen(string);
 	for(i = 0; i < len; i++) {
-		if((('A' <= uriString[i]) && (uriString[i] <= 'Z')) ||
-		   (('a' <= uriString[i]) && (uriString[i] <= 'z')) ||
-		   (('0' <= uriString[i]) && (uriString[i] <= '9')) ||
-		   (uriString[i] == '-') || 
-		   (uriString[i] == '_') ||
-		   (uriString[i] == '.') || 
-		   (uriString[i] == '?') || 
-		   (uriString[i] == '!') ||
-		   (uriString[i] == '~') ||
-		   (uriString[i] == '*') ||
-		   (uriString[i] == '\'') ||
-		   (uriString[i] == '(') ||
-		   (uriString[i] == ')'))
-		   	tmp = g_strdup_printf("%s%c", newURIString, uriString[i]);
-		else if(uriString[i] == ' ')
-			tmp = g_strdup_printf("%s%c", newURIString, '+');
-		else if((unsigned char)uriString[i] <= 127) {
-			tmp = g_strdup_printf(newURIString, hex = byte_to_hex(uriString[i]));g_free(hex);
+		if(g_ascii_isalnum(string[i]) || strchr("-_.!~*'()", (int)string[i]))
+		   	tmp = g_strdup_printf("%s%c", newURIString, string[i]);
+		else if(string[i] == ' ')
+			tmp = g_strdup_printf("%s%%20", newURIString);
+		else if((unsigned char)string[i] <= 127) {
+			tmp = g_strdup_printf(newURIString, hex = byte_to_hex(string[i]));g_free(hex);
 		} else {
 			bytes = 0;
-			if(((unsigned char)uriString[i] >= 192) && ((unsigned char)uriString[i] <= 223))
+			if(((unsigned char)string[i] >= 192) && ((unsigned char)string[i] <= 223))
 				bytes = 2;
-			else if(((unsigned char)uriString[i] > 223) && ((unsigned char)uriString[i] <= 239))
+			else if(((unsigned char)string[i] > 223) && ((unsigned char)string[i] <= 239))
 				bytes = 3;
-			else if(((unsigned char)uriString[i] > 239) && ((unsigned char)uriString[i] <= 247))
+			else if(((unsigned char)string[i] > 239) && ((unsigned char)string[i] <= 247))
 				bytes = 4;
-			else if(((unsigned char)uriString[i] > 247) && ((unsigned char)uriString[i] <= 251))
+			else if(((unsigned char)string[i] > 247) && ((unsigned char)string[i] <= 251))
 				bytes = 5;
-			else if(((unsigned char)uriString[i] > 247) && ((unsigned char)uriString[i] <= 251))
+			else if(((unsigned char)string[i] > 247) && ((unsigned char)string[i] <= 251))
 				bytes = 6;
 				
 			if(0 != bytes) {
@@ -504,27 +657,122 @@ gchar * encodeURIString(gchar *uriString) {
 				}
 
 				for(j=0; j < (bytes - 1); j++) {
-					tmp = g_strdup_printf("%s%s", newURIString, hex = byte_to_hex((unsigned char)uriString[i++]));
+					tmp = g_strdup_printf("%s%s", newURIString, hex = byte_to_hex((unsigned char)string[i++]));
 					g_free(hex);
 					g_free(newURIString);
 					newURIString = tmp;
 				}
-				tmp = g_strdup_printf("%s%s", newURIString, hex = byte_to_hex((unsigned char)uriString[i]));
+				tmp = g_strdup_printf("%s%s", newURIString, hex = byte_to_hex((unsigned char)string[i]));
 				g_free(hex);
 			} else {
-				// sh..!
-				g_error(_("Internal error while converting UTF-8 chars to HTTP URI!"));
+				/* sh..! */
+				g_error("Internal error while converting UTF-8 chars to HTTP URI!");
 			}
 		}
 		g_free(newURIString); 
 		newURIString = tmp;
 	}
-	g_free(uriString);
+	g_free(string);
 
 	return newURIString;
 }
 
+xmlChar * common_uri_escape(const xmlChar *url) {
+	xmlChar	*result;
+	
+	result = xmlURIEscape(url);
+	
+	/* workaround for libxml2 problem... */
+	if(NULL == result)
+		result = g_strdup(url);
+		
+	return result;	
+}
+
+/* to correctly escape and expand URLs, does not touch the
+   passed strings */
+xmlChar * common_build_url(const gchar *url, const gchar *baseURL) {
+	xmlChar	*escapedURL, *absURL, *escapedBaseURL;
+
+	escapedURL = common_uri_escape(url);
+
+	if(NULL != baseURL) {
+		escapedBaseURL = common_uri_escape(baseURL);	
+		absURL = xmlBuildURI(escapedURL, escapedBaseURL);
+		xmlFree(escapedURL);
+		xmlFree(escapedBaseURL);
+	} else {
+		absURL = escapedURL;
+	}
+	
+	return absURL;
+}
+
 gchar * filter_title(gchar * title) {
+
 	return g_strstrip(title);
 }
 
+#ifndef HAVE_STRSEP
+/* code taken from glibc-2.2.1/sysdeps/generic/strsep.c */
+char* strsep (char **stringp, const char *delim) {
+	char *begin, *end;
+
+	begin = *stringp;
+	if (begin == NULL)
+		return NULL;
+
+	/* A frequent case is when the delimiter string contains only one
+	   character.  Here we don't need to call the expensive `strpbrk'
+	   function and instead work using `strchr'.  */
+	if (delim[0] == '\0' || delim[1] == '\0')
+		{
+			char ch = delim[0];
+
+			if (ch == '\0')
+				end = NULL;
+			else
+				{
+					if (*begin == ch)
+						end = begin;
+					else if (*begin == '\0')
+						end = NULL;
+					else
+						end = strchr (begin + 1, ch);
+				}
+		}
+	else
+		/* Find the end of the token.  */
+		end = strpbrk (begin, delim);
+
+	if (end)
+		{
+			/* Terminate the token and set *STRINGP past NUL character.  */
+			*end++ = '\0';
+			*stringp = end;
+		}
+	else
+		/* No more delimiters; this is the last token.  */
+		*stringp = NULL;
+	return begin;
+}
+#endif /*HAVE_STRSEP*/
+
+/* Taken from gaim 24 June 2004, copyrighted by the gaim developers
+   under the GPL, etc.... */
+gchar *strreplace(const char *string, const char *delimiter,
+			   const char *replacement) {
+	gchar **split;
+	gchar *ret;
+
+	g_return_val_if_fail(string      != NULL, NULL);
+	g_return_val_if_fail(delimiter   != NULL, NULL);
+	g_return_val_if_fail(replacement != NULL, NULL);
+
+	split = g_strsplit(string, delimiter, 0);
+	ret = g_strjoinv(replacement, split);
+	g_strfreev(split);
+
+	return ret;
+}
+
