@@ -1,7 +1,8 @@
 /**
  * @file mozembed.c a browser module implementation using gtkmozembed.
  *   
- * Copyright (C) 2003 Lars Lindner <lars.lindner@gmx.net>   
+ * Copyright (C) 2003, 2004 Lars Lindner <lars.lindner@gmx.net>   
+ * Copyright (C) 2004 Nathan J. Conrad <t98502@users.sourceforge.net>
  *
  * Contains code from the Galeon sources
  *
@@ -31,11 +32,12 @@
 #include <errno.h>
 #include <glib.h>
 #include <gtkmozembed.h>
-#include "../htmlview.h"
+#include "../ui_htmlview.h"
 #include "../conf.h"
 #include "../support.h"
-#include "../callbacks.h"
-#include "../common.h"
+#include "../debug.h"
+#include "../ui_popup.h"
+#include "../ui_tabs.h"
 #include "mozilla.h"
 
 /* points to the URL actually under the mouse pointer or is NULL */
@@ -59,7 +61,7 @@ static void mozilla_write(GtkWidget *widget, const gchar *string, const gchar *b
 		   245960. Now off to eat dinner hours late because I missed
 		   the bus that I needed to use to get home. -Nathan*/
 		int left = strlen(string);
-		gtk_moz_embed_open_stream(GTK_MOZ_EMBED(widget), base, "text/html");
+		gtk_moz_embed_open_stream(GTK_MOZ_EMBED(widget), "file://", "text/html");
 		while (left > 0) {
 			if (left > 4096) {
 				gtk_moz_embed_append_data(GTK_MOZ_EMBED(widget), string, 4096);
@@ -71,6 +73,8 @@ static void mozilla_write(GtkWidget *widget, const gchar *string, const gchar *b
 		gtk_moz_embed_close_stream(GTK_MOZ_EMBED(widget));
 	} else
 		gtk_moz_embed_render_data(GTK_MOZ_EMBED(widget), EMPTY, strlen(EMPTY), base, "text/html");
+		
+	mozilla_scroll_to_top(widget);
 }
 
 /* -------------------------------------------------------------------- */
@@ -90,9 +94,10 @@ static void mozembed_new_window_cb(GtkMozEmbed *dummy, GtkMozEmbed **retval, gui
 	   correct (e.g. on initial webpage loading when new popups
 	   are requested and the user crosses a link)  */
 	if(NULL != selectedURL) {
-		/* FIXME: is it correct to assume that we always
-		   want to open new windows in the external browser? */
-		ui_htmlview_launch_in_external_browser(selectedURL);
+		if(getBooleanConfValue(BROWSE_INSIDE_APPLICATION))
+			ui_tabs_new(selectedURL, selectedURL);
+		else
+			ui_htmlview_launch_in_external_browser(selectedURL);
 	}
 	*retval = NULL;
 }
@@ -131,7 +136,7 @@ static gint mozembed_dom_mouse_click_cb (GtkMozEmbed *dummy, gpointer dom_event,
 		g_warning("Cannot determine mouse button!\n");
 		return FALSE;
 	}
-
+	
 	/* do we have a right mouse button click? */
 	if(button == 2) {
 		if(NULL == selectedURL)
@@ -142,7 +147,15 @@ static gint mozembed_dom_mouse_click_cb (GtkMozEmbed *dummy, gpointer dom_event,
 				       NULL, NULL, button, 0);
 	
 		return TRUE;
-	} else {	
+	/* or a middle button click */
+	} else if(button == 1) {
+		if(NULL != selectedURL) {	
+			ui_tabs_new(selectedURL, selectedURL);
+			return TRUE;
+		} else {
+			return FALSE;
+		}			
+	} else {
 		return FALSE;
 	}
 }
@@ -154,23 +167,19 @@ static gint mozembed_dom_mouse_click_cb (GtkMozEmbed *dummy, gpointer dom_event,
  * to keep the new document from being loaded. The uri argument is the
  * uri that's going to be loaded.
  */
-gint mozembed_open_uri_cb (GtkMozEmbed *embed, const char *uri, gpointer data) {
-
-	if(getBooleanConfValue(BROWSE_INSIDE_APPLICATION)) {
+static gint mozembed_open_uri_cb (GtkMozEmbed *embed, const char *uri, gpointer data) {
+	
+	if(GPOINTER_TO_INT(g_object_get_data(G_OBJECT(embed), "internal_browsing")) || getBooleanConfValue(BROWSE_INSIDE_APPLICATION)) {
 		return FALSE;
 	} else {
-		ui_htmlview_launch_in_external_browser(uri);
+		ui_htmlview_launch_URL(GTK_WIDGET(data), (gchar *)uri, UI_HTMLVIEW_LAUNCH_DEFAULT);
 		return TRUE;
 	}
 }
 
-void mozembed_destroy_brsr_cb (GtkMozEmbed *embed, gpointer data) {
-
-}
-
 /* Sets up a html view widget using GtkMozEmbed.
    The signal setting was derived from the Galeon source. */
-static GtkWidget * mozilla_create() {
+static GtkWidget * mozilla_create(gboolean forceInternalBrowsing) {
 	GtkWidget	*widget;
 	int		i;
 	
@@ -182,22 +191,22 @@ static GtkWidget * mozilla_create() {
 	}
 	signal_connections[] =
 	{
-		//{ "location",        mozembed_location_changed_cb  },
-		//{ "title",           mozembed_title_changed_cb     },
-		//{ "net_start",       mozembed_load_started_cb      },
-		//{ "net_stop",        mozembed_load_finished_cb     },
-		//{ "net_state_all",   mozembed_net_status_change_cb },
-		//{ "progress",        mozembed_progress_change_cb   },
+		/*{ "location",        mozembed_location_changed_cb  },*/
+		/*{ "title",           mozembed_title_changed_cb     },*/
+		/*{ "net_start",       mozembed_load_started_cb      },*/
+		/*{ "net_stop",        mozembed_load_finished_cb     },*/
+		/*{ "net_state_all",   mozembed_net_status_change_cb },*/
+		/*{ "progress",        mozembed_progress_change_cb   },*/
 		{ "link_message",    mozembed_link_message_cb      },
-		//{ "js_status",       mozembed_js_status_cb         },
-		//{ "visibility",      mozembed_visibility_cb        },
-		{ "destroy_browser", mozembed_destroy_brsr_cb      },
-		//{ "dom_mouse_down",  mozembed_dom_mouse_down_cb    },	
+		/*{ "js_status",       mozembed_js_status_cb         },*/
+		/*{ "visibility",      mozembed_visibility_cb        },*/
+		/*{ "destroy_browser", mozembed_destroy_brsr_cb      },*/
+		/*{ "dom_mouse_down",  mozembed_dom_mouse_down_cb    },*/
 		{ "dom_mouse_click", mozembed_dom_mouse_click_cb   },
 		{ "dom_key_press",   mozembed_dom_key_press_cb     },
-		//{ "size_to",         mozembed_size_to_cb           },
+		/*{ "size_to",         mozembed_size_to_cb           },*/
 		{ "new_window",      mozembed_new_window_cb        },
-		//{ "security_change", mozembed_security_change_cb   },
+		/*{ "security_change", mozembed_security_change_cb   },*/
 		{ "open_uri",		 mozembed_open_uri_cb},
 		/* terminator -- must be last in the list! */
 		{ NULL, NULL } 
@@ -215,7 +224,9 @@ static GtkWidget * mozilla_create() {
 						widget);
 						
 	}
-
+	
+	g_object_set_data(G_OBJECT(widget), "internal_browsing", GINT_TO_POINTER(forceInternalBrowsing));
+	
 	return widget;
 }
 
@@ -233,9 +244,19 @@ static void mozilla_init() {
 	/* initialize profile */
 	gtk_moz_embed_set_profile_path(profile, "liferea");
 	g_free(profile);
-
+	
 	/* startup done */
 	gtk_moz_embed_push_startup();
+	
+	mozilla_preference_set_boolean("javascript.enabled", !getBooleanConfValue(DISABLE_JAVASCRIPT));
+	mozilla_preference_set_boolean("plugin.default_plugin_disabled", FALSE);
+	mozilla_preference_set_boolean("xpinstall.enabled", FALSE);
+	mozilla_preference_set_boolean("mozilla.widget.raise-on-setfocus", FALSE);
+	mozilla_save_prefs();
+}
+
+static void mozilla_deinit() {
+	gtk_moz_embed_pop_startup();
 }
 
 /* launches the specified URL */
@@ -246,17 +267,32 @@ static void launch_url(GtkWidget *widget, const gchar *url) {
 
 static gboolean launch_inside_possible(void) { return TRUE; }
 
+static void mozilla_set_proxy(gchar *hostname, int port, gchar *username, gchar *password) {
+	if(NULL != hostname) {
+		debug0(DEBUG_GUI, "setting proxy for Mozilla");
+		mozilla_preference_set("network.proxy.http", hostname);
+		mozilla_preference_set_int("network.proxy.http_port", port);
+		mozilla_preference_set_int("network.proxy.type", 1);
+	} else {
+		mozilla_preference_set_int("network.proxy.type", 0);
+	}
+	
+	mozilla_save_prefs();
+}
+
 static htmlviewPluginInfo mozillaInfo = {
 	HTMLVIEW_API_VERSION,
 	"Mozilla",
 	mozilla_init,
+	mozilla_deinit,
 	mozilla_create,
 	mozilla_write,
 	launch_url,
 	launch_inside_possible,
 	mozilla_get_zoom,
 	mozilla_set_zoom,
-	mozilla_scroll_pagedown
+	mozilla_scroll_pagedown,
+	mozilla_set_proxy
 };
 
 DECLARE_HTMLVIEW_PLUGIN(mozillaInfo);
