@@ -1,38 +1,38 @@
-/*
-   search GUI dialogs
-   
-   Copyright (C) 2003 Lars Lindner <lars.lindner@gmx.net>
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
+/**
+ * @file ui_search.c everything about searching
+ *
+ * Copyright (C) 2003, 2004 Lars Lindner <lars.lindner@gmx.net>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
 
 #include "callbacks.h"
 #include "interface.h"
 #include "feed.h"
 #include "folder.h"
-#include "filter.h"
+#include "rule.h"
+#include "vfolder.h"
 #include "support.h"
 #include "common.h"
 
 extern GtkWidget 	*mainwindow;
 static GtkWidget 	*feedsterdialog = NULL;
-
-extern feedPtr          allItems;
+static feedPtr		searchFeed = NULL;
 
 /*------------------------------------------------------------------------------*/
-/* search dialog callbacks								*/
+/* search dialog callbacks							*/
 /*------------------------------------------------------------------------------*/
 
 /* called when toolbar search button is clicked */
@@ -59,7 +59,6 @@ void on_hidesearch_clicked(GtkButton *button, gpointer user_data) {
 	}
 }
 
-
 void on_searchentry_activate(GtkButton *button, gpointer user_data) {
 	GtkWidget		*searchentry;
 	G_CONST_RETURN gchar	*searchstring;
@@ -67,8 +66,33 @@ void on_searchentry_activate(GtkButton *button, gpointer user_data) {
 	g_assert(mainwindow != NULL);
 	if(NULL != (searchentry = lookup_widget(mainwindow, "searchentry"))) {
 		searchstring = gtk_entry_get_text(GTK_ENTRY(searchentry));
-		ui_mainwindow_set_status_bar(_("searching for \"%s\""), searchstring);
-		ui_itemlist_load(allItems, (gchar *)searchstring);
+		ui_mainwindow_set_status_bar(_("Searching for \"%s\""), searchstring);
+		ui_itemlist_clear();
+		if(NULL != searchFeed)
+			feed_free(searchFeed);
+		searchFeed = vfolder_new();
+		feed_set_title(searchFeed, searchstring);
+		vfolder_add_rule(searchFeed, "exact", searchstring, TRUE);
+		vfolder_refresh(searchFeed);
+		ui_feedlist_select(NULL);
+		itemlist_load((nodePtr)searchFeed);
+	}
+}
+
+void on_newVFolder_clicked(GtkButton *button, gpointer user_data) {
+	gint			pos;
+	feedPtr			fp;
+	folderPtr		folder = NULL;
+	
+	if(NULL != searchFeed) {
+		folder = ui_feedlist_get_target_folder(&pos);
+		fp = searchFeed;
+		searchFeed = NULL;
+		ui_feedlist_add(folder, (nodePtr)fp, pos);
+		ui_feedlist_update();
+		ui_feedlist_select((nodePtr)fp);
+	} else {
+		ui_show_info_box(_("Please do a search first!"));
 	}
 }
 
@@ -86,8 +110,8 @@ void on_feedsterbtn_clicked(GtkButton *button, gpointer user_data) {
 	if((NULL != keywords) && (NULL != resultCountButton)) {
 		resultCount = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(resultCountButton));
 		searchtext = (gchar *)g_strdup(gtk_entry_get_text(GTK_ENTRY(keywords)));
-		searchtext = encodeURIString(searchtext);
-		searchtext = g_strdup_printf("http://www.feedster.com/rss.php?q=%s&sort=date&type=rss&ie=UTF-8&limit=%d", 
+		searchtext = encode_uri_string(searchtext);
+		searchtext = g_strdup_printf("http://www.feedster.com/search.php?q=%s&sort=date&type=rss&ie=UTF-8&limit=%d", 
 					    searchtext, (int)gtk_adjustment_get_value(resultCount));
 
 		ui_feedlist_new_subscription(searchtext, NULL, FALSE);
@@ -107,43 +131,3 @@ void on_search_with_feedster_activate(GtkMenuItem *menuitem, gpointer user_data)
 	gtk_widget_show(feedsterdialog);
 }
 
-void on_newVFolder_clicked(GtkButton *button, gpointer user_data) {
-	GtkWidget		*searchentry;
-	G_CONST_RETURN gchar	*searchstring;
-//	rulePtr			rp;	// FIXME: this really does not belong here!!! -> vfolder.c
-//	feedPtr			fp;
-	folderPtr			folder = NULL;
-
-	g_assert(mainwindow != NULL);
-		
-	if(NULL != (searchentry = lookup_widget(mainwindow, "searchentry"))) {
-		searchstring = gtk_entry_get_text(GTK_ENTRY(searchentry));
-		ui_mainwindow_set_status_bar(_("creating VFolder for search term \"%s\""), searchstring);
-
-		//ptr = (nodePtr)ui_feedlist_get_target_folder();
-		
-		if(NULL != folder) {
-
-//			if(NULL != (fp = feed_add(FST_VFOLDER, "", folder, "untitled",NULL,0,FALSE))) {
-				
-				// FIXME: this really does not belong here!!! -> vfolder.c
-				/* setup a rule */
-//				rp = g_new0(struct rule,1);
-
-				/* we set the searchstring as a default title */
-//				feed_set_title(fp, (gpointer)g_strdup_printf(_("VFolder %s"),searchstring));
-				/* and set the rules... */
-/*				rp->value = g_strdup((gchar *)searchstring);
-				setVFolderRules(fp, rp);*/
-
-				/* FIXME: brute force: update of all vfolders redundant */
-//				loadVFolders();
-				
-//				ui_folder_add_feed(fp, FALSE);
-//			}
-		} else {
-			g_warning("internal error! could not get folder key prefix!");
-		}
-	}
-}
-
